function imBlurW3 = EnhanceBlur(a)
%% Initial Processing

% Get the Image
%a = imread('007595.jpg');

% Convert to Grayscale and get the size
im = rgb2gray(a);
imSize = size(im);
numRows = imSize(1);
numCols = imSize(2);

% Use Retinex filtering to remove soft shadows
imRetinex = retinex_frankle_mccann(double(im)./255, 60);
imRetinex = imfilter(imRetinex, fspecial('gaussian',3,.5));

%% Find Edges and Create Initial Blur Map
imEdge = edge(imRetinex, 'canny');
imEdge = uint8(imEdge);
imEdge = 255.*imEdge;
imBlurMap = imfilter(imEdge, fspecial('gaussian',27,6));
imBlurMap = imAdjust(imBlurMap);

%% Find Big Block Area using the Initial Blur Map
maxVal = 0;
minVal = numRows*numCols*255;

for i =10:10:100
    area = round((min(imSize)/i));
    if mod(area,2) ~= 0 
        area = area+1;
    end
    offSet = area/2;
    imbox = imfilter(imBlurMap,fspecial('average',offSet));
    
    if sum(sum(imbox))>maxVal
        maxVal = sum(sum(imbox));
        maxOff = offSet;
        maxI = i;
    end
    
    if sum(sum(imbox))<minVal
        minVal = sum(sum(imbox));
        minOff = offSet;
        minI = i;
    end
end

imboxMin = imadjust(imfilter(imBlurMap, fspecial('average',minOff)));
imboxMax = imadjust(imfilter(imBlurMap, fspecial('average',maxOff)));
imDiff = imadjust((imboxMax/2)+imboxMin);

% Find 1/6 of image with the most edges
imAvg = imfilter(imDiff, fspecial('gaussian', floor(min(imSize)/8),27)); % was 6
imAvg = imadjust(imAvg);
imNew = imboxMin;
for rows = 1:numRows
    for cols = 1:numCols
        if imAvg(rows,cols) > 140
            imNew(rows, cols) =imNew(rows, cols) +80; % was 150
        end
    end
end

imNew = imfilter(imNew, fspecial('gaussian', 6, 3));

%% Generate the blur map to be used for filtering

growBy =8;

%Make the final image bigger
imInverted = uint8(zeros(numRows+(growBy*2), numCols+(growBy*2), 1));

% Duplicate top two rows
imInverted(1:2,growBy+1:numCols+growBy,:) = 255;

% Duplicate bottom two rows
imInverted(numRows+growBy:numRows+4,growBy+1:numCols+growBy,:) = 255;

% Duplicate Left Side
imInverted(growBy+1:numRows+growBy,1:2,:) = 255;

% Duplicate Right Side20
imInverted(growBy+1:numRows+growBy,numCols+1:numCols+growBy+1,:) = 255;

% Paste in the center
imInverted(growBy+1:numRows+growBy, growBy+1:numCols+growBy, :)= imNew(:,:,:);

imInverted = InvertIm(imInverted);

%% Generate the final image

%Make the final image bigger
imFinal = uint8(zeros(numRows+(growBy*2), numCols+(growBy*2), 3));

% Duplicate top rows
imFinal(1:growBy,growBy+1:numCols+growBy,:) = a(1:growBy,:,:);

% Duplicate bottom two rows
imFinal(numRows+growBy:numRows+(growBy*2),growBy+1:numCols+growBy,:) = a(numRows-growBy:numRows,:,:);

% Duplicate Left Side
imFinal((growBy+1):numRows+growBy,1:growBy,:) = a(:,1:growBy,:);

% Duplicate Right Side
imFinal((growBy+1):numRows+growBy,numCols+growBy:numCols+((2*growBy)),:) = a(:,numCols-growBy:numCols,:);

% Paste in the center
imFinal((growBy+1):numRows+growBy, (growBy+1):numCols+growBy, :)= a(:,:,:);

% Sharpen the image
imFinal = deconvreg(imFinal, fspecial('gaussian', 3));

% Apply Blur
for bLevel = .1:.01: 1
    imFinalBlurMap = imInverted;
    imFinalBlurMap = imFinalBlurMap-((255*bLevel)-1);
    imFinalBlurMap = logical(imFinalBlurMap);
    % Blur Each Color Channel
    for i=1:3
        imFinal(:, :, i) = roifilt2(fspecial('disk', (bLevel*1.5)), imFinal(:, :, i), imFinalBlurMap);
    end
end

% Crop Image back to original size
imFinal = imFinal((growBy+1):numRows+growBy,(growBy+1):numCols+growBy, :);

% Crop Blur map back to original size
imInverted = imInverted((growBy+1):numRows+growBy,(growBy+1):numCols+growBy, :);

% Convert blur map to a 3 channel image
imBlurW3 = uint8(zeros(numRows, numCols, 3));
for i = 1:3
    imBlurW3(:, : , i) = imInverted(:, :, 1);
end

% Show image results
%figure, imshow([a,imBlurW3, imFinal]); title 'Original Image - Blur Map - Resulting Image';
